/**
 * @fileoverview A simple organizer for ordering hooks.
 * @author Hiukky
 */
'use strict';
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
module.exports = {
    meta: {
        docs: {
            description: 'A simple organizer for ordering hooks.',
            category: 'Non-matching declaration order.',
            url: 'https://github.com/hiukky/eslint-plugin-hooks-sort/blob/master/docs/rules/order.md',
            recommended: false,
        },
        fixable: undefined,
        schema: [
            {
                type: 'object',
                properties: {
                    groups: {
                        type: 'array',
                    },
                },
            },
        ],
    },
    create: function (ctx) {
        var options = ctx.options[0];
        var orderHooks = [];
        return {
            /**
             * @function VariableDeclaration
             *
             * @param {TNode} node
             */
            VariableDeclaration: function (node) {
                var declaration = node.declarations[0].init;
                if ((declaration === null || declaration === void 0 ? void 0 : declaration.type) === 'CallExpression' &&
                    node.kind === 'const' &&
                    declaration.callee.name) {
                    if (declaration.callee.name.slice(0, 3) === 'use') {
                        orderHooks.push([declaration.callee.name, node]);
                    }
                }
            },
            /**
             * @function Program
             *
             * @param {TNode} node
             */
            'Program:exit': function () {
                var groups = (options === null || options === void 0 ? void 0 : options.groups) || [
                    'useContext',
                    'useState',
                    'useEffect',
                ];
                var matchingHooks = __spreadArrays(orderHooks).filter(function (hook) {
                    return groups.includes(hook[0]);
                });
                var orderHooksCorrect = __spreadArrays(matchingHooks).sort(function (a, b) { return groups.indexOf(a[0]) - groups.indexOf(b[0]); });
                matchingHooks.filter(function (hook, index) {
                    if (orderHooksCorrect.length > 1 &&
                        orderHooksCorrect[index][0] !== hook[0]) {
                        ctx.report(hook[1], "Non-matching declaration order. " + hook[0] + " comes " + (!index ? 'after' : 'before') + " " + orderHooksCorrect[index][0] + ".");
                    }
                });
            },
        };
    },
};
//# sourceMappingURL=order.js.map